//////////////////////////////////////////////////////////////////////////////
//
// PocketFrog - The Game Library for Pocket PC Devices
// Copyright 2002  Thierry Tremblay
//
// Permission to use, copy, modify, distribute and sell this software
// and its documentation for any purpose is hereby granted without fee,
// provided that the above copyright notice appear in all copies and
// that both that copyright notice and this permission notice appear
// in supporting documentation.  Thierry Tremblay makes no representations
// about the suitability of this software for any purpose.
// It is provided "as is" without express or implied warranty.
//
//////////////////////////////////////////////////////////////////////////////

#include "image.h"
#include "frogdefs.h"
#include "pocketpc.h"
#include "memfile.h"
#include "pockethal.h"
#include <cassert>

#include "imgdecmp.h"
#pragma comment( lib, "imgdecmp.lib" )


//////////////////////////////////////////////////////////////////////////////
//
// CreateSurface() - Create a surface from a bitmap
//
//////////////////////////////////////////////////////////////////////////////

static Surface* CreateSurface(HBITMAP hBitmap )
{
   // Retrieve bitmap info
   BITMAP bitmap;
   ::GetObject( hBitmap, sizeof(BITMAP), &bitmap );

   // Create the surface
   GDISurface* surface = new GDISurface( bitmap.bmWidth, bitmap.bmHeight );

   // Blit the bitmap on the surface, doing color conversion if needed
   HDC hDestDC = surface->GetDC();
   HDC hSrcDC  = ::CreateCompatibleDC( hDestDC );

   ::SelectObject( hSrcDC, hBitmap );
   ::BitBlt( hDestDC, 0, 0, bitmap.bmWidth, bitmap.bmHeight, hSrcDC, 0, 0, SRCCOPY );

   ::DeleteDC( hSrcDC );
   ::DeleteDC( hDestDC );

   return surface;
}


//////////////////////////////////////////////////////////////////////////////
//
// Callbacks for DecompressImageIndirect()
//
//////////////////////////////////////////////////////////////////////////////

static DWORD CALLBACK LoadImage( char* buffer, DWORD dwBufferMax, LPARAM lparam )
{
   // We already have locked or memory mapped the image, simply return it's size.
   return dwBufferMax;
}



static void CALLBACK LoadImageProgress( IImageRender* pRender, BOOL bComplete, LPARAM lparam )
{
   // We don't care about the load progress...
}



//////////////////////////////////////////////////////////////////////////////
//
// DecompressImage() - Invoke imgdecmp.dll to decompress the image
//
//////////////////////////////////////////////////////////////////////////////

static Surface* DecompressImage(const uint8_t* pBegin, const uint8_t* pEnd )
{
   // This will be filled with a handle to the image
   HBITMAP hBitmap = 0;    

   // Prepare the decompression process
   // Here I use 24 bits output because if I select 16, imgdecmp.dll will
   // create a 555 DIB instead of a 565 (darn).
   DecompressImageInfo dii;

   dii.dwSize                = sizeof(dii);       // Size of this structure
   dii.pbBuffer              = (BYTE*)pBegin;     // Pointer to the buffer to use for data
   dii.dwBufferMax           = pEnd - pBegin;     // Size of the buffer
   dii.dwBufferCurrent       = 0;                 // The amount of data which is current in the buffer
   dii.phBM                  = &hBitmap;          // Pointer to the bitmap returned (can be NULL)
   dii.ppImageRender         = 0;                 // Pointer to an IImageRender object (can be NULL)
   dii.iBitDepth             = 24;                // Bit depth of the output image
   dii.lParam                = 0;                 // User parameter for callback functions
   dii.hdc                   = 0;                 // HDC to use for retrieving palettes
   dii.iScale                = 100;               // Scale factor (1 - 100)
   dii.iMaxWidth             = 8192;              // Maximum width of the output image
   dii.iMaxHeight            = 8192;              // Maxumum height of the output image
   dii.pfnGetData            = LoadImage;         // Callback function to get image data
   dii.pfnImageProgress      = LoadImageProgress; // Callback function to notify caller of progress decoding the image
   dii.crTransparentOverride = -1;                // If this color is not (UINT)-1, it will override the
                                                  // transparent color in the image with this color. (GIF ONLY)

   // Process and decompress the image data
   HRESULT hr = ::DecompressImageIndirect( &dii );
   if (FAILED(hr)) return 0;

   Surface* surface = CreateSurface(hBitmap );
   ::DeleteObject( hBitmap );

   return surface;
}



//////////////////////////////////////////////////////////////////////////////
//
// LoadImage()
//
//////////////////////////////////////////////////////////////////////////////

Surface* LoadImage(const TCHAR* filename )
{
    Internal::MemoryFile file( filename );
    if (!file.IsOpen())
        return 0;
    
    return DecompressImage(file.begin(), file.end() );
}



Surface* LoadImage(unsigned resourceID, const TCHAR* resclass, HINSTANCE hModule )
{
    // If the module instance is not specified, get the application's instance
    if (!hModule) hModule = ::GetModuleHandle(0);
    
    // If the class is RT_BITMAP, imgdecmp.dll won't be able to decompress it as the file header
    // is not present (BITMAPFILEHEADER). In any case, it's better to use LoadBitmap()
    if (resclass == RT_BITMAP)
    {
        HBITMAP hBitmap = ::LoadBitmap( hModule, (LPCTSTR)resourceID );
        if (!hBitmap)
            return 0;
        
        Surface* surface = CreateSurface( hBitmap );
        ::DeleteObject( hBitmap );
        
        return surface;
    }
    else
    {
        // Lock the resource in memory
        HRSRC hResource = ::FindResource( hModule, (LPCTSTR)resourceID, resclass );
        if (!hResource)
        {
            // Try RT_BITMAP
            if (resclass != RT_BITMAP)
                return LoadImage( resourceID, RT_BITMAP, hModule );
            
            return 0;
        }
        
        DWORD    size    = ::SizeofResource( hModule, hResource );
        HGLOBAL  hMemory = ::LoadResource( hModule, hResource );
        uint8_t* data    = (uint8_t*)::LockResource( hMemory );
        
        // Decompress the image
        return DecompressImage( data, data + size );
    }
}



Surface* LoadImage( const uint8_t* pBegin, const uint8_t* pEnd )
{
    return DecompressImage( pBegin, pEnd );
}



bool SaveImage( const Surface* const_surface, const TCHAR* filename )
{
    Surface* surface = const_cast<Surface*>( const_surface );
    
    if (!surface || !filename)
        return false;

    TCHAR pathname[MAX_PATH];
    PocketPC::GetFullPathName( filename, pathname );
    _tcscat( pathname, _T(".bmp") );   

  
    // Lines must be dword aligned
    assert( ((surface->GetWidth() * sizeof(Pixel))&3)==0 );
    
    BITMAPINFOHEADER info;
    info.biSize          = sizeof(BITMAPINFOHEADER);
    info.biWidth         = surface->GetWidth();
    info.biHeight        = surface->GetHeight();
    info.biPlanes        = 1;
    info.biBitCount      = 16;
    info.biCompression   = BI_BITFIELDS;
    info.biSizeImage     = info.biWidth * info.biHeight;
    info.biXPelsPerMeter = 0;
    info.biYPelsPerMeter = 0;
    info.biClrUsed       = 0;
    info.biClrImportant  = 0; 
    
    uint32_t colors[3] = { RED_MASK, GREEN_MASK, BLUE_MASK };
    
    
    BITMAPFILEHEADER header;
    header.bfType      = 0x4d42;
    header.bfSize      = sizeof(BITMAPFILEHEADER) + info.biSize + sizeof(colors) + info.biSizeImage;
    header.bfReserved1 = 0;
    header.bfReserved2 = 0;
    header.bfOffBits   = sizeof(BITMAPFILEHEADER) + info.biSize + sizeof(colors);
    
    
    // Create the file
    HANDLE hFile = ::CreateFile( pathname, GENERIC_WRITE, 0, 0, CREATE_ALWAYS, 0, 0 );
    if (hFile == INVALID_HANDLE_VALUE)
    {
        return false;
    }
    
    DWORD temp;
    
    // Write the header + bitmap info
    ::WriteFile( hFile, &header, sizeof(header), &temp, 0 );
    ::WriteFile( hFile, &info, sizeof(info), &temp, 0 );
    ::WriteFile( hFile, colors, sizeof(colors), &temp, 0 );
    
    // Write the image (must flip image vertically)
    uint16_t* pixels = surface->GetPixels();
    pixels = (uint16_t*)((uint8_t*)pixels + surface->GetPitch() * (surface->GetHeight()-1));
    
    for (int h = surface->GetHeight(); h; --h)
    {
        ::WriteFile( hFile, pixels, surface->GetWidth() * sizeof(uint16_t), &temp, 0 );
        pixels = (Pixel*)((uint8_t*)pixels - surface->GetPitch());
    }
    
    ::CloseHandle( hFile );
    
    return true;
}

